@@ -1,7 +1,7 @@
 """
 file path manipulation
 
-Copyright: 2011-2018 Ethan Furman
+Copyright: 2011-2019 Ethan Furman
 """
 
 from os import F_OK, R_OK, W_OK, X_OK
@@ -10,7 +10,7 @@ import os as _os
 import shutil as _shutil
 import sys as _sys
 
-__all__ = ['Path', 'F_OK', 'R_OK', 'W_OK', 'X_OK']
+__all__ = ['Path', 'F_OK', 'R_OK', 'W_OK', 'X_OK', 'ospath']
 
 _py_ver = _sys.version_info[:2]
 
@@ -38,14 +38,21 @@ class Path(object):
     ext   = .part1.ext
     """
 
-    def __new__(cls, string=''):
-        if isinstance(string, cls):
-            return string
-        if isinstance(string, unicode):
+    def __new__(cls, *paths):
+        paths = tuple([ospath(p) for p in paths])
+        if not paths:
+            paths = (unicode(), )
+        elif len(paths) == 1 and isinstance(paths[0], cls):
+            return paths[0]
+        if isinstance(paths[0], unicode):
+            str_cls = unicode
             new_cls = uPath
         else:
+            str_cls = bytes
             new_cls = bPath
-        p = new_cls.__new__(new_cls, string)
+        if not all_equal(paths, test=lambda p: isinstance(p, str_cls)):
+            raise TypeError('invalid path types: %r' % ([type(p) for p in paths], ))
+        p = new_cls.__new__(new_cls, *paths)
         return p
 
     @staticmethod
@@ -68,13 +75,27 @@ class Path(object):
             return cls.getcwd()
 
     @staticmethod
-    def glob(pattern):
+    def glob(pattern=None):
+        if pattern is None:
+            pattern = uPath._STAR
         return [Path(p) for p in native_glob(pattern)]
 
     @staticmethod
-    def listdir(dir):
+    def listdir(dir=None):
+        if dir is None:
+            dir = uPath._DOT
         return [Path(p) for p in _os.listdir(dir)]
 
+    @staticmethod
+    def abspath(name):
+        "can result in invalid path if symlinks are present"
+        return Path(_os.path.abspath(name))
+
+    @staticmethod
+    def absolute(name):
+        'returns path from root without resolving .. dirs'
+        return Path(name).absolute()
+
     @staticmethod
     def access(file_name, mode):
         return Path(file_name).access(mode)
@@ -85,7 +106,7 @@ class Path(object):
 
     @staticmethod
     def chdir(subdir):
-        Path(subdir).chdir()
+        return Path(subdir).chdir()
 
     if _py_ver >= (2, 6) and not _is_win:
         @classmethod
@@ -116,6 +137,27 @@ class Path(object):
         def chroot(subdir):
             return Path(subdir).chroot()
 
+    @staticmethod
+    def commonprefix(*paths):
+        if not paths:
+            return Path(unicode())
+        elif len(paths) == 1 and not isinstance(paths[0], list):
+            return Path(paths[0])
+        elif (
+                len(paths) > 1 and isinstance(paths[0], list) or
+                not all_equal(paths, test=lambda x: type(x))
+                ):
+            raise TypeError('invalid path types: %r' % ([type(p) for p in paths], ))
+        else:
+            paths = [Path(p).elements for p in paths]
+            common = []
+            for prefixes in zip(*paths):
+                if all_equal(prefixes):
+                    common.append(prefixes[0])
+                else:
+                    break
+            return Path(*common)
+
     @classmethod
     def copy(cls, files, dst):
         """
@@ -138,21 +180,39 @@ class Path(object):
     def exists(file_name):
         return Path(file_name).exists()
 
+    @staticmethod
+    def expanduser(name='~'):
+        return Path(_os.path.expanduser(name))
+
+    @staticmethod
+    def expandvars(name):
+        return Path(_os.path.expandvars(name))
+
+    @staticmethod
+    def isabs(name):
+        'thin wrapper os.path.isabs()'
+        raise NotImplementedError()
+
+    @staticmethod
+    def isabsolute(name):
+        'os.path.isabs + windows paths must have drive-letter'
+        raise NotImplementedError()
+
     @staticmethod
     def isdir(name):
-        return Path(name).isdir()
+        return _os.path.isdir(name)
 
     @staticmethod
     def isfile(name):
-        return Path(name).isfile()
+        return _os.path.isfile(name)
 
     @staticmethod
     def islink(name):
-        return Path(name).islink()
+        return _os.path.islink(name)
 
     @staticmethod
     def ismount(name):
-        return Path(name).ismount()
+        return _os.path.ismount(name)
 
     @staticmethod
     def iter_all(name):
@@ -221,10 +281,16 @@ class Path(object):
 
     @classmethod
     def move(cls, sources, dst):
+        dst = Path(dst)
         if isinstance(sources, cls.basecls):
             sources = Path.glob(sources)
         for source in sources:
             Path(source).move(dst)
+        return dst
+
+    @staticmethod
+    def normcase(path):
+        return Path(_os.path.normcase(path))
 
     @staticmethod
     def open(name, mode='r', buffering=None, encoding=None):
@@ -241,6 +307,16 @@ class Path(object):
 
         readlink = _os.readlink
 
+    @staticmethod
+    def realcase(path):
+        "noop on posix, actual case of path on nt"
+        raise NotImplementedError()
+
+    @staticmethod
+    def realpath(path):
+        "return canonical path (all symlinks resolved)"
+        return Path(_os.path.realpath(path))
+
     @classmethod
     def removedirs(cls, subdirs):
         if isinstance(subdirs, cls.basecls):
@@ -270,6 +346,10 @@ class Path(object):
         for subdir in subdirs:
             Path(subdir).rmtree(ignore_errors=ignore_errors, onerror=onerror)
 
+    @staticmethod
+    def samefile(path1, path2):
+        return _os.path.samefile(path1, path2)
+
     @staticmethod
     def stat(name):
         return Path(name).stat()
@@ -321,29 +401,50 @@ Path.basecls = bytes, str, unicode
 
 class Methods(object):
 
-    def __new__(cls, string=None):
+    def __new__(cls, *paths):
         base_cls = cls.basecls[1]       # bytes or unicode
-        if string is None:
-            string = base_cls()
+        if not paths:
+            paths = (base_cls(), )
+        elif len(paths) > 1:
+            new_paths = []
+            for first, second in zip(paths[:-1], paths[1:]):
+                if second.startswith(('/', cls._SYS_SEP)):
+                    new_paths[:] = []
+                    continue
+                new_paths.append(first.rstrip('/'))
+            new_paths.append(second)
+            paths = tuple(new_paths)
         slash = cls._SLASH
-        vol = dirs = filename = base = ext = base_cls()
+        string = slash.join(paths)
+        vol = dirs = filename = base = ext = protocol = host = site = params = fragments = base_cls()
         if cls._SYS_SEP != '/':
             string = string.replace(cls._SYS_SEP, slash)
-        if string[:2] == slash+slash:           # usually '//'
-            pieces = string.split(slash)
+        pieces = string.split(slash)
+        if string[:2] == slash+slash and string[2:3] != slash:           # usually '//'
+            if len(pieces) < 4:
+                raise ValueError('bad path: %r' % string)
             vol = slash.join(pieces[:4])
             pieces = pieces[4:]
             if pieces:
                 pieces.insert(0, cls._EMPTY)
         elif string[1:2] == cls._COLON and _os.path.__name__ == 'ntpath':
-            vol, string = string[:2], string[2:]
-            pieces = string.split(slash)
+            vol = pieces.pop(0)
         else:
             vol = cls._EMPTY
-            pieces = string.split(slash)
-        for bit in pieces[1:-1]:
-            if not bit:
-                raise ValueError("bad path: %r" % string)
+        if len(pieces) > 2 and pieces[0].endswith(cls._COLON) and not pieces[1]:
+            protocol = pieces[0] + cls._SLASH + cls._SLASH
+            if len(pieces) > 3:
+                host = pieces[2]
+            site = protocol + host
+            pieces = pieces[2:]
+            if cls._QUESTION in string:
+                params = string.split(cls._QUESTION)[1].split(cls._HASHTAG)[0]
+                params = params.split(cls._AMPERSAND)
+                params = dict([p.split(cls._EQUALS) for p in params])
+            if cls._HASHTAG in string:
+                fragments = tuple(string.split(cls._HASHTAG)[1].split(cls._AMPERSAND))
+        if len(pieces) > 2:
+            pieces = pieces[:1] + [p for p in pieces[1:-1] if p] + pieces[-1:]
         if pieces:
             if pieces[-1] in (cls._CUR_DIR, cls._PREV_DIR, cls._EMPTY):
                 dirs = slash.join(pieces)
@@ -357,45 +458,101 @@ class Methods(object):
                     base, ext = filename[:ext_start], filename[ext_start:]
                 else:
                     base = filename
-        p = base_cls.__new__(cls, vol + dirs + filename)
+        p = base_cls.__new__(cls, protocol + vol + dirs + filename)
         p._vol = vol
         p._dirs = dirs
-        p._path = vol + dirs
+        p._dirname = vol + dirs
         p._filename = filename
         p._base = base
         p._ext = ext
+        p._protocol = protocol
+        p._host = host
+        p._site = site
+        p._parameters = params
+        p._fragments = fragments
         return p
 
+    @property
+    def protocol(self):
+        return self._protocol[:-3]
+
+    @property
+    def host(self):
+        return self._host
+
+    @property
+    def site(self):
+        return self._site
+
+    @property
+    def parameters(self):
+        return self._parameters
+
+    @property
+    def fragments(self):
+        return self._fragments
+
     @property
     def vol(self):
+        'volume/drive of path'
         return self.__class__(self._vol)
+    drive = vol
+
+    @property
+    def root(self):
+        if self._dirs[:1] == self._SLASH or self._vol[:1] == self._SLASH:
+            return self.__class__(self._SLASH)
+        else:
+            return self._EMPTY
+
+    @property
+    def anchor(self):
+        'pathlib: drive+root'
+        return self.drive + self.root
 
     @property
     def dirs(self):
+        'directories without volume/drive'
         result = self.__class__(self._dirs)
         if len(result) > 1:
-            result = result.rstrip('/')
+            result = result.rstrip(self._SLASH)
         return result
 
+    @property
+    def parent(self):
+        'first half of os.path.split(...)'
+        return self.__class__(self._vol + self._dirs)
+    dirname = parent
+
     @property
     def path(self):
-        return self._vol + self.dirs
+        'self - for compatibility with stdlib'
+        return self
 
     @property
     def filename(self):
+        'second half of os.path.split(...)'
         return self.__class__(self._filename)
+    name = basename = filename
 
     @property
     def base(self):
         return self.__class__(self._base)
+    stem = base
 
     @property
     def ext(self):
         return self.__class__(self._ext)
+    suffix = ext
+
+    @property
+    def suffixes(self):
+        return [self.__class__(e) for e in self._filename.split('.')[1:]]
 
     @property
     def elements(self):
         return list(self.iter_all())
+    parts = elements
 
     @property
     def dir_elements(self):
@@ -404,22 +561,22 @@ class Methods(object):
     def __add__(self, other):
         if not isinstance(other, self.basecls):
             return NotImplemented
-        return Path(self._path + self._filename + other)
+        return Path(self._dirname + self._filename + other)
 
     def __contains__(self, text):
         text = text.replace(self._SYS_SEP, self._SLASH)
-        return text in self._path+self._filename
+        return text in self._dirname+self._filename
 
     def __div__(self, other):
         if not isinstance(other, self.basecls):
             return NotImplemented
         other = Path(other)
         current = self.__class__()
-        if other._vol:
+        if other._vol or other._protocol:
             if self:
                 raise ValueError("Cannot combine %r and %r" % (self, other))
             # current = other._vol
-        current += self._path + self._filename
+        current += self._protocol + self._dirname + self._filename
         if current[-1:] == self._SLASH:
             current = current[:-1]
         next = other._dirs + other._filename
@@ -432,15 +589,19 @@ class Methods(object):
         if not isinstance(other, self.basecls):
             return NotImplemented
         other = Path(other)
-        return self._path == other._path and self._filename == other._filename
+        return self._protocol == other._protocol and self._dirname == other._dirname and self._filename == other._filename
 
     def __hash__(self):
-        return (self._path + self._filename).__hash__()
+        return (self._protocol + self._dirname + self._filename).__hash__()
 
     def __mod__(self, other):
-        return Path((self._path + self._filename) % other)
+        return Path((self._protocol + self._dirname + self._filename) % other)
 
     def __mul__(self, other):
+        """
+        if other.vol, self is ignored;
+        if other.protocol, other.protocol is ignored
+        """
         if not isinstance(other, self.basecls):
             return NotImplemented
         other = Path(other)
@@ -448,6 +609,7 @@ class Methods(object):
             vol = other._vol
             current = []
         else:
+            protocol = self._protocol
             vol = self._vol
             current = self.dir_elements
             if self._filename:
@@ -473,17 +635,19 @@ class Methods(object):
         dirs = self._SLASH.join(new_path)
         if dirs[-1:] != self._SLASH:
             dirs += self._SLASH
-        if vol[:2] == self._SLASH*2 and dirs[:1] != self._SLASH:
+        if (vol[:2] == self._SLASH*2 or protocol) and dirs[:1] != self._SLASH:
             dirs = self._SLASH + dirs
-        return Path(self._EMPTY.join([vol, dirs, filename]))
+        return Path(self._EMPTY.join([vol or protocol, dirs, filename]))
 
     def __ne__(self, other):
+        if not isinstance(other, self.basecls):
+            return NotImplemented
         return not self == other
 
     def __radd__(self, other):
         if not isinstance(other, self.basecls):
             return NotImplemented
-        return Path(other + self._path + self._filename)
+        return Path(other + self._protocol + self._dirname + self._filename)
 
     def __rdiv__(self, other):
         if not isinstance(other, self.basecls):
@@ -493,11 +657,14 @@ class Methods(object):
     __rtruediv__ = __rdiv__
 
     def __repr__(self):
-        string = self._path + self._filename
+        string = ''
+        if self._protocol:
+            string = self._protocol
+        string += self._dirname + self._filename
         return "Path(%r)" % string
 
     def __rmod__(self, other):
-        return other % (self._path + self._filename)
+        return other % (self._protocol + self._dirname + self._filename)
 
     def __rmul__(self, other):
         if not isinstance(other, self.basecls):
@@ -512,7 +679,10 @@ class Methods(object):
         return other - self
 
     def __str__(self):
-        string = self._path + self._filename
+        string = ''
+        if self._protocol:
+            string = self._protocol
+        string += self._dirname + self._filename
         return string
 
     def __sub__(self, other):
@@ -521,7 +691,7 @@ class Methods(object):
         other = Path(other)
         if other == self._EMPTY:
             return self
-        if other._vol != self._vol:
+        if other._vol != self._vol or other._protocol != self._protocol:
             raise ValueError("cannot subtract %r from %r" % (other, self))
         vol = self._EMPTY
         o = other._dirs + other._filename
@@ -558,6 +728,7 @@ class Methods(object):
             subdir = self/subdir
         subdir = base_class(subdir)
         _os.chdir(subdir)
+        return subdir
 
     if (2, 6) <= _py_ver < (3, 3) and not _is_win:
 
@@ -659,9 +830,11 @@ class Methods(object):
     if not _is_win:
         def chroot(self, subdir=None):
             if subdir is None:
-                return _os.chroot(self)
+                subdir = self
             else:
-                return _os.chroot(self/subdir)
+                subdir = self/subdir
+            _os.chroot(self/subdir)
+            return subdir
 
     def copy(self, files, dst=None):
         """
@@ -698,7 +871,7 @@ class Methods(object):
         new_sub = sub.replace(self._SYS_SEP, self._SLASH)
         start = start or 0
         end = end or len(self)
-        return (self._path + self._filename).count(new_sub)
+        return (self._protocol + self._dirname + self._filename).count(new_sub)
 
     def descend(self):
         pieces = self.elements
@@ -720,7 +893,7 @@ class Methods(object):
                 raise TypeError("Can't convert %r implicitly" % suffix.__class__)
         start = start or 0
         end = end or len(self)
-        return (self._path + self._filename).endswith(new_suffix, start, end)
+        return (self._protocol + self._dirname + self._filename).endswith(new_suffix, start, end)
 
     def exists(self, name=None):
         if name is not None:
@@ -732,7 +905,7 @@ class Methods(object):
         new_sub = sub.replace(self._SYS_SEP, self._SLASH)
         start = start or 0
         end = end or len(self)
-        return (self._path + self._filename).find(new_sub)
+        return (self._protocol + self._dirname + self._filename).find(new_sub)
 
     def format(self, other):
         raise AttributeError("'Path' object has no attribute 'format'")
@@ -741,16 +914,23 @@ class Methods(object):
         raise AttributeError("'Path' object has no attribute 'format_map'")
 
     def glob(self, pattern=None):
-        if pattern is None:
-            pattern = self
-        else:
+        if self and pattern:
             pattern = self/pattern
+        elif self and (self._STAR in self or self._QUESTION in self):
+                pattern = self
+        elif self:
+                pattern = self / self._STAR
+        elif pattern:
+            pass
+        else:
+            pattern = self._STAR
         return [Path(p) for p in native_glob(pattern)]
 
     def index(self, sub, start=None, end=None):
         result = self.find(sub, start, end)
         if result == -1:
             raise ValueError('substring not found')
+        return result
 
     def isdir(self, name=None):
         if name is not None:
@@ -858,9 +1038,15 @@ class Methods(object):
             return _os.link(source, new_name)
 
     def listdir(self, subdir=None):
-        if subdir is not None:
-            self /= subdir
-        return [Path(p) for p in _os.listdir(self)]
+        if self and subdir:
+            subdir = self / subdir
+        elif self:
+            subdir = self
+        elif subdir:
+            pass
+        else:
+            subdir = self.__class__(self._DOT)
+        return [Path(p) for p in _os.listdir(subdir)]
 
     if hasattr(_os, 'lstat'):
 
@@ -873,7 +1059,7 @@ class Methods(object):
     def lstrip(self, chars=None):
         if chars is not None:
             chars = chars.replace(self._SYS_SEP, self._SLASH)
-        return self.__class__((self._path + self._filename).lstrip(chars))
+        return self.__class__((self._protocol + self._dirname + self._filename).lstrip(chars))
 
     if hasattr(_os, 'mkfifo'):
 
@@ -958,6 +1144,7 @@ class Methods(object):
         for file in files:
             src = base_class(file)
             _shutil.move(src, dst)
+        return dst
 
     def open(self, file_name=None, mode=None, buffering=None, encoding=None):
         """
@@ -1026,6 +1213,7 @@ class Methods(object):
             file_name = self/file_name
         src, dst = base_class(file_name, dst)
         _os.rename(src, dst)
+        return dst
 
     def renames(self, file_name, dst=None):
         if dst is None:
@@ -1040,9 +1228,9 @@ class Methods(object):
         old = old.replace(self._SYS_SEP, self._SLASH)
         new = new.replace(self._SYS_SEP, self._SLASH)
         if count:
-            return self.__class__((self._path + self._filename).replace(old, new, count))
+            return self.__class__((self._protocol + self._dirname + self._filename).replace(old, new, count))
         else:
-            return self.__class__((self._path + self._filename).replace(old, new))
+            return self.__class__((self._protocol + self._dirname + self._filename).replace(old, new))
 
     def rmdir(self, subdirs=None):
         'thin wrapper around os.rmdir'
@@ -1083,7 +1271,7 @@ class Methods(object):
     def rstrip(self, chars=None):
         if chars is not None:
             chars = chars.replace(self._SYS_SEP, self._SLASH)
-        return self.__class__((self._path + self._filename).rstrip(chars))
+        return self.__class__((self._protocol + self._dirname + self._filename).rstrip(chars))
 
     def startswith(self, prefix, start=None, end=None):
         if isinstance(prefix, self.basecls):
@@ -1095,7 +1283,7 @@ class Methods(object):
                 raise TypeError("Can't convert %r to %s implicitly" % (prefix.__class__, self.__class__.__name__))
         start = start or 0
         end = end or len(self)
-        return (self._path + self._filename).startswith(new_prefix, start, end)
+        return (self._protocol + self._dirname + self._filename).startswith(new_prefix, start, end)
 
     def stat(self, file_name=None):
         if file_name is not None:
@@ -1114,7 +1302,7 @@ class Methods(object):
     def strip(self, chars=None):
         if chars is not None:
             chars = chars.replace(self._SYS_SEP, self._SLASH)
-        return self.__class__((self._path + self._filename).strip(chars))
+        return self.__class__((self._protocol + self._dirname + self._filename).strip(chars))
 
     def strip_ext(self, remove=1):
         remove_all = False
@@ -1123,7 +1311,7 @@ class Methods(object):
             remove = -1
         while (remove_all or remove > 0) and self.ext:
             remove -= 1
-            self = self.__class__(self._path + self._base)
+            self = self.__class__(self._protocol + self._dirname + self._base)
         return self
 
     if not _is_win:
@@ -1134,7 +1322,8 @@ class Methods(object):
             else:
                 source = self/source
             source, new_name = base_class(source, new_name)
-            return _os.symlink(source, new_name)
+            _os.symlink(source, new_name)
+            return new_name
 
     def unlink(self, files=None):
         "thin wrapper around os.unlink"
@@ -1195,6 +1384,11 @@ class bPath(Methods, Path, bytes):
     _PREV_DIR = '..'.encode('ascii')
     _SLASH = '/'.encode('ascii')
     _SYS_SEP = system_sep.encode('ascii')
+    _QUESTION = '?'.encode('ascii')
+    _HASHTAG = '#'.encode('ascii')
+    _AMPERSAND = '&'.encode('ascii')
+    _EQUALS = '='.encode('ascii')
+    _STAR = '*'.encode('ascii')
 
 class uPath(Methods, Path, unicode):
     _COLON = unicode(':')
@@ -1204,6 +1398,11 @@ class uPath(Methods, Path, unicode):
     _PREV_DIR = unicode('..')
     _SLASH = unicode('/')
     _SYS_SEP = unicode(system_sep)
+    _QUESTION = unicode('?')
+    _HASHTAG = unicode('#')
+    _AMPERSAND = unicode('&')
+    _EQUALS = unicode('=')
+    _STAR = unicode('*')
 
 if _py_ver < (3, 0):
     bPath.basecls = bPath, bytes, uPath, unicode
@@ -1212,7 +1411,6 @@ else:
     bPath.basecls = bPath, bytes
     uPath.basecls = uPath, unicode
 
-# add xmlrpc support
 if _py_ver < (3, 0):
     from xmlrpclib import Marshaller
     Marshaller.dispatch[bPath] = Marshaller.dump_string
@@ -1234,3 +1432,27 @@ def base_class(*paths):
         return result[0]
     else:
         return tuple(result)
+
+def all_equal(iterator, test=None):
+    '''if `test is None` do a straight equality test'''
+    it = iter(iterator)
+    if test is None:
+        try:
+            target = next(it)
+            test = lambda x: x == target
+        except StopIteration:
+            return True
+    for item in it:
+        if not test(item):
+            return False
+    return True
+
+
+def ospath(thing):
+    try:
+        return thing.__ospath__()
+    except AttributeError:
+        if isinstance(thing, (bytes, unicode)):
+            return thing
+        raise TypeError('%r must be a bytes, str, or path-like object, not %r' % (thing, type(thing)))
+
